@@ -91,6 +91,34 @@ void Document_Glom::set_connection_database(const Glib::ustring& strVal)
   }
 }
 
+void Document_Glom::set_relationship(const Glib::ustring& table_name, const Relationship& relationship)
+{
+  //Find the existing relationship:
+  type_tables::iterator iterFind = m_tables.find(table_name);
+  if(iterFind != m_tables.end())
+  {
+    DocumentTableInfo& info = iterFind->second;
+
+    //Look for the relationship with this name:
+    bool existing = false;
+    const Glib::ustring relationship_name = relationship.get_name();
+
+    for(type_vecRelationships::iterator iter = info.m_relationships.begin(); iter != info.m_relationships.end(); ++iter)
+    {
+      if(iter->get_name() == relationship_name)
+      {
+        *iter = relationship;
+        existing = true;
+      } 
+    }
+
+    if(!existing)
+    {
+      //Add a new one if it's not there.
+      info.m_relationships.push_back(relationship);
+    }
+  }
+}
 
 bool Document_Glom::get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name, Relationship& relationship) const
 {
@@ -101,7 +129,7 @@ bool Document_Glom::get_relationship(const Glib::ustring& table_name, const Glib
   if(iterFind != m_tables.end())
   {
     const DocumentTableInfo& info = iterFind->second;
-    
+
     //Look for the relationship with this name:
     for(type_vecRelationships::const_iterator iter = info.m_relationships.begin(); iter != info.m_relationships.end(); ++iter)
     {
@@ -109,10 +137,10 @@ bool Document_Glom::get_relationship(const Glib::ustring& table_name, const Glib
       {
         relationship = *iter;
         return true; //found
-      }        
+      }
     }
   }
-   
+
   return false; //Not found.
 }
 
@@ -137,6 +165,68 @@ void Document_Glom::set_relationships(const Glib::ustring& table_name, const typ
   }
 }
 
+void Document_Glom::remove_relationship(const Relationship& relationship)
+{
+  //Get the table that this relationship is part of:
+  type_tables::iterator iter = m_tables.find(relationship.get_from_table());
+  if(iter != m_tables.end())
+  {
+    DocumentTableInfo& info = iter->second;
+
+    //Find the relationship and remove it:
+    for(type_vecRelationships::iterator iter = info.m_relationships.begin(); iter != info.m_relationships.end(); ++iter)
+    {
+      if(iter->get_name() == relationship.get_name())
+      {
+        iter = info.m_relationships.erase(iter);
+
+        //TODO: Remove any lookups, view fields, or related records portals, that use this relationship.
+
+        set_modified(true);
+      }
+    }
+  }
+}
+
+
+void Document_Glom::remove_table(const Glib::ustring& table_name)
+{
+  type_tables::iterator iter = m_tables.find(table_name);
+  if(iter != m_tables.end())
+  {
+    m_tables.erase(iter);
+    set_modified(true);
+  }
+
+  //Remove any relationships that use this table:
+  for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
+  {
+    DocumentTableInfo& info = iter->second;
+
+    bool something_changed = true;
+    type_vecRelationships::iterator iterRel = info.m_relationships.begin();
+    while(something_changed)
+    {
+      if(iterRel->get_to_table() == table_name)
+      {
+        //Loop again, because we have changed the structure:
+        remove_relationship(*iterRel); //Also removes anything that uses the relationship.
+
+        something_changed = true;
+        iterRel = info.m_relationships.begin();
+      }
+      else
+      {
+        if(iterRel == info.m_relationships.end())
+          something_changed = false; //We've looked at them all, without changing things.
+      }
+    }
+
+  }
+
+}
+
+
 Document_Glom::type_vecFields Document_Glom::get_table_fields(const Glib::ustring& table_name) const
 {
   if(!table_name.empty())
@@ -148,7 +238,7 @@ Document_Glom::type_vecFields Document_Glom::get_table_fields(const Glib::ustrin
       {
          g_warning("Document_Glom::get_table_fields: table found, but m_fields is empty");
       }
-      
+
       return iterFind->second.m_fields;
     }
     else
@@ -159,7 +249,7 @@ Document_Glom::type_vecFields Document_Glom::get_table_fields(const Glib::ustrin
   }
   else
   {
-    g_warning("Document_Glom::get_table_fields: table name is empty.");
+    //g_warning("Document_Glom::get_table_fields: table name is empty.");
     return type_vecFields();
   }
 }
@@ -212,7 +302,7 @@ void Document_Glom::change_field_name(const Glib::ustring& table_name, const Gli
       iterFind->set_name(strFieldNameNew);
     }
     
-    //Find any relationships or layouts that use this fieldt
+    //Find any relationships or layouts that use this field
     //Look at each table:
     for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
     {
@@ -227,7 +317,7 @@ void Document_Glom::change_field_name(const Glib::ustring& table_name, const Gli
             iterRels->set_from_field(strFieldNameNew);
           }
         }
-        
+
         if(iterRels->get_to_table() == table_name)
         {
           if(iterRels->get_to_field() == strFieldNameOld)
@@ -235,10 +325,10 @@ void Document_Glom::change_field_name(const Glib::ustring& table_name, const Gli
             //Change it:
             iterRels->set_to_field(strFieldNameNew);
           }
-        }               
+        }
       }
     }
-    
+
     //Look at each layout:
     //TODO: Remember to change it in other layouts when we add the ability to show fields from other tables.
     for(DocumentTableInfo::type_layouts::iterator iterLayouts = iterFindTable->second.m_layouts.begin(); iterLayouts != iterFindTable->second.m_layouts.end(); ++iterLayouts)
@@ -349,7 +439,7 @@ Document_Glom::type_listTableInfo Document_Glom::get_tables() const
   {
     result.push_back(iter->second.m_info);
   }
-        
+
   return result;
 }
 
@@ -362,9 +452,9 @@ void Document_Glom::set_tables(const type_listTableInfo& tables)
   for(type_tables::iterator iter = m_tables.begin(); iter != m_tables.end(); iter++)
   {
     const DocumentTableInfo& doctableinfo = iter->second;
-    
+
     const Glib::ustring table_name = doctableinfo.m_info.m_name;
-  
+
     type_listTableInfo::const_iterator iterfind = std::find_if(tables.begin(), tables.end(), predicate_FieldHasName<TableInfo>(table_name));
     if(iterfind != tables.end())
     {
@@ -382,6 +472,12 @@ void Document_Glom::set_tables(const type_listTableInfo& tables)
   
 }
 
+Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_relationship_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Relationship& relationship) const
+{
+  //TODO: Use an actual relationship_name instead of concatenating:
+  return get_data_layout_groups_plus_new_fields(layout_name, relationship.get_from_table() + "_related_" + relationship.get_name()); 
+}
+
 Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& table_name) const
 {
   type_mapLayoutGroupSequence result = get_data_layout_groups(layout_name, table_name);
@@ -392,7 +488,7 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
   if(result.empty())
     create_default = true;
   //TODO: Also set create_default is all groups have no fields.
-  
+
   if(create_default)
   {
     //Get the last top-level group. We will add new fields into this one:
@@ -419,27 +515,30 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
     for(type_vecFields::const_iterator iter = all_fields.begin(); iter != all_fields.end(); ++iter)
     {
       const Glib::ustring field_name = iter->get_name();
-
-      //See whether it's already in the result:
-      //TODO_Performance: There is a lot of iterating and comparison here:
-      bool found = false; //TODO: This is horrible.
-      for(type_mapLayoutGroupSequence::const_iterator iterFind = result.begin(); iterFind != result.end(); ++iterFind)
+      if(!field_name.empty())
       {
-        if(iterFind->second.has_field(field_name))
+        //See whether it's already in the result:
+        //TODO_Performance: There is a lot of iterating and comparison here:
+        bool found = false; //TODO: This is horrible.
+        for(type_mapLayoutGroupSequence::const_iterator iterFind = result.begin(); iterFind != result.end(); ++iterFind)
         {
-          found = true;
-          break;
+          if(iterFind->second.has_field(field_name))
+          {
+            found = true;
+            break;
+          }
         }
-      }
 
-      if(!found)
-      {
-        LayoutItem_Field layout_item;
-        layout_item.set_name(field_name);
-        //layout_item.m_sequence = sequence;  add_item() will fill this.
-        layout_item.m_hidden = false;
+        if(!found)
+        {
+          LayoutItem_Field layout_item;
+          layout_item.set_name(field_name);
+          //layout_item.set_table_name(table_name); //TODO: Allow viewing of fields through relationships.
+          //layout_item.m_sequence = sequence;  add_item() will fill this.
 
-        pTopLevel->add_item(layout_item);
+
+          pTopLevel->add_item(layout_item);
+        }
       }
     }
   }
@@ -453,7 +552,7 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
   if(iterFind != m_tables.end())
   {
     const DocumentTableInfo& info = iterFind->second;
-    
+
     //Look for the layout with this name:
     DocumentTableInfo::type_layouts::const_iterator iter = info.m_layouts.find(layout_name);
     if(iter != info.m_layouts.end())
@@ -461,10 +560,16 @@ Document_Glom::type_mapLayoutGroupSequence Document_Glom::get_data_layout_groups
       return iter->second; //found   
     }
   }
-     
+
   return type_mapLayoutGroupSequence(); //not found
 }
 
+void Document_Glom::set_relationship_data_layout_groups(const Glib::ustring& layout_name, const Relationship& relationship, const type_mapLayoutGroupSequence& groups)
+{
+  //TODO: Use an actual relationship_name instead of concatenating:
+  set_data_layout_groups(layout_name, relationship.get_from_table() + "_related_" + relationship.get_name(), groups);
+}
+
 void Document_Glom::set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& table_name, const type_mapLayoutGroupSequence& groups)
 {
   if(!table_name.empty())
@@ -520,8 +625,18 @@ bool Document_Glom::get_table_is_known(const Glib::ustring& table_name) const
 
 AppState::userlevels Document_Glom::get_userlevel() const
 {
+  userLevelReason reason;
+  return get_userlevel(reason);
+}
+
+AppState::userlevels Document_Glom::get_userlevel(userLevelReason& reason) const
+{
+  //Initialize output parameter:
+  reason = USER_LEVEL_REASON_UNKNOWN;
+
   if(get_read_only())
   {
+    reason = USER_LEVEL_REASON_FILE_READ_ONLY;
     return AppState::USERLEVEL_OPERATOR; //A read-only document can not be changed, so there's no point in being in developer mode. This is one way to control the user level on purpose.
   }
   else if(m_file_uri.empty()) //If it has never been saved then this is a new default document, so the user created it, so the user can be a developer.
@@ -544,9 +659,19 @@ void Document_Glom::on_app_state_userlevel_changed(AppState::userlevels userleve
   m_signal_userlevel_changed.emit(userlevel);
 }
 
-void Document_Glom::set_userlevel(AppState::userlevels userlevel)
+bool Document_Glom::set_userlevel(AppState::userlevels userlevel)
 {
-  m_app_state.set_userlevel(userlevel);
+  //Prevent incorrect user level:
+  if((userlevel == AppState::USERLEVEL_DEVELOPER) && get_read_only())
+  {
+    m_app_state.set_userlevel(AppState::USERLEVEL_OPERATOR);
+    return false;
+  }
+  else
+  { 
+    m_app_state.set_userlevel(userlevel);
+    return true;
+  }
 }
 
 void Document_Glom::emit_userlevel_changed()
@@ -601,18 +726,18 @@ void Document_Glom::set_modified(bool value)
   }
 }
 
-void Document_Glom::load_after_layout_group(const xmlpp::Element* node, LayoutGroup& group)
+void Document_Glom::load_after_layout_group(const xmlpp::Element* node, const Glib::ustring table_name, LayoutGroup& group)
 {
   if(!node)
     return;
-    
+
   //Get the group details:
   group.set_name( get_node_attribute_value(node, "name") );
   group.m_title = get_node_attribute_value(node, "title");
   group.m_columns_count = get_node_attribute_value_as_decimal(node, "columns_count");
 
   group.m_sequence = get_node_attribute_value_as_decimal(node, "sequence");
-  
+
   //Get the child items:
   xmlpp::Node::NodeList listNodes = node->get_children(); 
   for(xmlpp::Node::NodeList::iterator iter = listNodes.begin(); iter != listNodes.end(); ++iter)
@@ -621,12 +746,15 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, LayoutGr
     if(element)
     {
       const guint sequence = get_node_attribute_value_as_decimal(element, "sequence");
-      
+
       if(element->get_name() == "data_layout_item")
       {
         LayoutItem_Field item;
 
         item.set_name( get_node_attribute_value(element, "name") );
+        item.set_relationship_name( get_node_attribute_value(element, "relationship") );
+        item.set_editable( get_node_attribute_value_as_bool(element, "editable") );
+        //item.set_table_name(table_name);
 
         item.m_sequence = sequence;
         group.add_item(item, sequence);
@@ -635,7 +763,7 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, LayoutGr
       {
         LayoutGroup child_group;
         //Recurse:
-        load_after_layout_group(element, child_group);
+        load_after_layout_group(element, table_name, child_group);
 
         group.add_item(child_group);
       }
@@ -648,7 +776,7 @@ void Document_Glom::load_after_layout_group(const xmlpp::Element* node, LayoutGr
         group.add_item(item, sequence);
       }
     }
-  }                          
+  }
 }
 
 bool Document_Glom::load_after()
@@ -661,7 +789,7 @@ bool Document_Glom::load_after()
     if(nodeRoot)
     {
       m_database_title = get_node_attribute_value(nodeRoot, "database_title");
-          
+
       const xmlpp::Element* nodeConnection = get_node_child_named(nodeRoot, "connection");
       if(nodeConnection)
       {
@@ -670,10 +798,10 @@ bool Document_Glom::load_after()
         m_connection_user = get_node_attribute_value(nodeConnection, "user");
         m_connection_database = get_node_attribute_value(nodeConnection, "database");
       }
-  
+
       //Tables:
       m_tables.clear();
- 
+
       //Look at each "table" node.
       xmlpp::Node::NodeList listNodes = nodeRoot->get_children("table");
       for(xmlpp::Node::NodeList::const_iterator iter = listNodes.begin(); iter != listNodes.end(); iter++)
@@ -681,26 +809,26 @@ bool Document_Glom::load_after()
         xmlpp::Element* nodeTable = dynamic_cast<xmlpp::Element*>(*iter);
         if(nodeTable)
         {
-         
+
           const Glib::ustring table_name = get_node_attribute_value(nodeTable, "name");
-          
+
           m_tables[table_name] = DocumentTableInfo();
           DocumentTableInfo& doctableinfo = m_tables[table_name]; //Setting stuff directly in the reference is more efficient than copying it later:
-            
+
           TableInfo table_info;
           table_info.m_name = table_name;
           table_info.m_hidden = get_node_attribute_value_as_bool(nodeTable, "hidden");
           table_info.m_title = get_node_attribute_value(nodeTable, "title");
           table_info.m_default = get_node_attribute_value_as_bool(nodeTable, "default");
-          
+
           doctableinfo.m_info = table_info;
-          
+
           //Fields:
           const xmlpp::Element* nodeFields = get_node_child_named(nodeTable, "fields");
           if(nodeFields)
           {
             const Field::type_map_type_names type_names = Field::get_type_names();
-            
+
             //Loop through Field child nodes:
             xmlpp::Node::NodeList listNodes = nodeFields->get_children("field");
             for(xmlpp::Node::NodeList::const_iterator iter = listNodes.begin(); iter != listNodes.end(); iter++)
@@ -709,17 +837,17 @@ bool Document_Glom::load_after()
               if(nodeChild)
               {
                 Field field;
-                
+
                 Gnome::Gda::FieldAttributes field_info = field.get_field_info();
                 const Glib::ustring strName = get_node_attribute_value(nodeChild, "name");
                 field_info.set_name( strName );
-                
+
                 const Glib::ustring strTitle = get_node_attribute_value(nodeChild, "title");
                 field.set_title(strTitle);
-                
+ 
                 field_info.set_primary_key( get_node_attribute_value_as_bool(nodeChild, "primary_key") );
                 field_info.set_unique_key( get_node_attribute_value_as_bool(nodeChild, "unique") );
-                field_info.set_auto_increment( get_node_attribute_value_as_bool(nodeChild, "auto_increment") );           
+                field_info.set_auto_increment( get_node_attribute_value_as_bool(nodeChild, "auto_increment") );
 
                 //Get lookup information, if present.
                 xmlpp::Element* nodeLookup = get_node_child_named(nodeChild, "field_lookup");
@@ -730,10 +858,10 @@ bool Document_Glom::load_after()
                 }
 
                 field.set_calculation( get_node_attribute_value(nodeChild, "calculation") );
-                
+
                 //Field Type:
                 const Glib::ustring field_type = get_node_attribute_value(nodeChild, "type");
-      
+
                 //Get the type enum for this string representation of the type:
                 Field::glom_field_type field_type_enum = Field::TYPE_INVALID;
                 for(Field::type_map_type_names::const_iterator iter = type_names.begin(); iter !=type_names.end(); ++iter)
@@ -744,7 +872,7 @@ bool Document_Glom::load_after()
                     break;
                   }
                 }
-                        
+
                 //Default value:
                 const Glib::ustring default_value_text = get_node_attribute_value(nodeChild, "default_value");
                 //Interpret the text as per the field type:
@@ -754,12 +882,12 @@ bool Document_Glom::load_after()
 
                 //We set this after set_field_info(), because that gets a glom type from the (not-specified) gdatype. Yes, that's strange, and should probably be more explicit.
                 field.set_glom_type( field_type_enum );
-                             
+ 
                 doctableinfo.m_fields.push_back(field);
               }
             }
           }
-           
+
           //Relationships:
           const xmlpp::Element* nodeRelationships = get_node_child_named(nodeTable, "relationships");
           if(nodeRelationships)
@@ -772,18 +900,23 @@ bool Document_Glom::load_after()
               {
                 Relationship relationship;
                 const Glib::ustring relationship_name = get_node_attribute_value(nodeChild, "name");
-                
+
                 relationship.set_from_table( table_name );
-                relationship.set_name( relationship_name );
+                relationship.set_name( relationship_name );;
+
+                const Glib::ustring relationship_title = get_node_attribute_value(nodeChild, "title");
+                relationship.set_title( relationship_title );
+
                 relationship.set_from_field( get_node_attribute_value(nodeChild, "key") );
                 relationship.set_to_table( get_node_attribute_value(nodeChild, "other_table") );
                 relationship.set_to_field( get_node_attribute_value(nodeChild, "other_key") );
-          
+                relationship.set_auto_create( get_node_attribute_value_as_bool(nodeChild, "auto_create") );
+
                 doctableinfo.m_relationships.push_back(relationship);
               }
             }
           }
-          
+
           //Layouts:
           const xmlpp::Element* nodeDataLayouts = get_node_child_named(nodeTable, "data_layouts");
           if(nodeDataLayouts)
@@ -795,9 +928,9 @@ bool Document_Glom::load_after()
               if(node)
               {
                 const Glib::ustring layout_name = get_node_attribute_value(node, "name");
-               
+
                 type_mapLayoutGroupSequence layout_groups;
-                         
+
                 const xmlpp::Element* nodeGroups = get_node_child_named(node, "data_layout_groups");
                 if(nodeGroups)
                 {
@@ -812,20 +945,61 @@ bool Document_Glom::load_after()
                       if(!group_name.empty())
                       {
                         LayoutGroup group;
-                        load_after_layout_group(node, group);
+                        load_after_layout_group(node, table_name, group);
 
                         layout_groups[group.m_sequence] = group;
                       }
                     }
                   }
                 }
-                
+
                 doctableinfo.m_layouts[layout_name] = layout_groups;
               }
             }
           } //if(nodeDataLayouts)
 
-        }
+
+          //Groups:
+          m_groups.clear();
+
+          const xmlpp::Element* nodeGroups = get_node_child_named(nodeRoot, "groups");
+          if(nodeGroups)
+          {
+            xmlpp::Node::NodeList listNodes = nodeGroups->get_children("group");
+            for(xmlpp::Node::NodeList::iterator iter = listNodes.begin(); iter != listNodes.end(); ++iter)
+            {
+              xmlpp::Element* node = dynamic_cast<xmlpp::Element*>(*iter);
+              if(node)
+              {
+                GroupInfo group_info;
+
+                group_info.m_name = get_node_attribute_value(node, "name");
+                group_info.m_developer = get_node_attribute_value_as_bool(node, "developer");
+
+                xmlpp::Node::NodeList listTablePrivs = nodeGroups->get_children("table_privs");
+                for(xmlpp::Node::NodeList::iterator iter = listTablePrivs.begin(); iter != listTablePrivs.end(); ++iter)
+                {
+                  xmlpp::Element* node = dynamic_cast<xmlpp::Element*>(*iter);
+                  if(node)
+                  {
+                    const Glib::ustring table_name = get_node_attribute_value(node, "table_name");
+
+                    Privileges privs;
+                    privs.m_view = get_node_attribute_value_as_bool(node, "priv_view");
+                    privs.m_edit = get_node_attribute_value_as_bool(node, "priv_edit");
+                    privs.m_create = get_node_attribute_value_as_bool(node, "priv_create");
+                    privs.m_delete = get_node_attribute_value_as_bool(node, "priv_delete");
+
+                    group_info.m_map_privileges[table_name] = privs;
+                  }
+                }
+
+                m_groups[group_info.m_name] = group_info;
+              }
+            }
+          }
+
+        } //root
       }
     }
   }
@@ -836,7 +1010,7 @@ bool Document_Glom::load_after()
 void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutGroup& group)
 {
   xmlpp::Element* child = node->add_child("data_layout_group");
-  
+
   child->set_attribute("name", group.get_name());
   child->set_attribute("title", group.m_title);
   set_node_attribute_value_as_decimal(child, "columns_count", group.m_columns_count);
@@ -861,6 +1035,8 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
       {
         xmlpp::Element* nodeItem = child->add_child("data_layout_item");
         nodeItem->set_attribute("name", item->get_name());
+        nodeItem->set_attribute("relationship", field->get_relationship_name());
+        set_node_attribute_value_as_bool(nodeItem, "editable", item->get_editable());
 
         set_node_attribute_value_as_decimal(nodeItem, "sequence", item->m_sequence);
       }
@@ -875,9 +1051,9 @@ void Document_Glom::save_before_layout_group(xmlpp::Element* node, const LayoutG
           set_node_attribute_value_as_decimal(nodeItem, "sequence", item->m_sequence);
         }
       }
-        
+
     }
-  }        
+  } 
 }
 
 bool Document_Glom::save_before()
@@ -886,17 +1062,17 @@ bool Document_Glom::save_before()
   if(nodeRoot)
   {
     set_node_attribute_value(nodeRoot, "database_title", m_database_title);
-    
+
     xmlpp::Element* nodeConnection = get_node_child_named_with_add(nodeRoot, "connection");
     set_node_attribute_value(nodeConnection, "server", m_connection_server); 
     set_node_attribute_value(nodeConnection, "user", m_connection_user); 
     set_node_attribute_value(nodeConnection, "database", m_connection_database);
-    
+
     //Remove existing tables:
     xmlpp::Node::NodeList listNodes = nodeRoot->get_children("table");
     for(xmlpp::Node::NodeList::iterator iter = listNodes.begin(); iter != listNodes.end(); ++iter)
       nodeRoot->remove_child(*iter);
-    
+
     //Add tables:
     for(type_tables::const_iterator iter = m_tables.begin(); iter != m_tables.end(); ++iter)
     {
@@ -904,7 +1080,7 @@ bool Document_Glom::save_before()
 
       if(doctableinfo.m_info.m_name.empty())
         g_warning("Document_Glom::save_before(): table name is empty.");
-        
+
       if(!doctableinfo.m_info.m_name.empty())
       {
         xmlpp::Element* nodeTable = nodeRoot->add_child("table");
@@ -961,9 +1137,11 @@ bool Document_Glom::save_before()
 
           xmlpp::Element* elemRelationship = elemRelationships->add_child("relationship");
           set_node_attribute_value(elemRelationship, "name", relationship.get_name());
+          set_node_attribute_value(elemRelationship, "title", relationship.get_title());
           set_node_attribute_value(elemRelationship, "key", relationship.get_from_field());
           set_node_attribute_value(elemRelationship, "other_table", relationship.get_to_table());
           set_node_attribute_value(elemRelationship, "other_key", relationship.get_to_field());
+          set_node_attribute_value_as_bool(elemRelationship, "auto_create", relationship.get_auto_create());
         }
 
 
@@ -985,10 +1163,45 @@ bool Document_Glom::save_before()
           }
         }
       }
-      
+
     } //for m_tables
+
+
+    //Remove existing groups:
+    listNodes = nodeRoot->get_children("groups");
+    for(xmlpp::Node::NodeList::iterator iter = listNodes.begin(); iter != listNodes.end(); ++iter)
+      nodeRoot->remove_child(*iter);
+
+    //Add groups:
+    xmlpp::Element* nodeGroups = nodeRoot->add_child("groups");
+
+    nodeGroups->add_child_comment("These are only used when recreating a database from an example file. The actualy access-control is on the server, of course.");
+
+    for(type_map_groups::const_iterator iter = m_groups.begin(); iter != m_groups.end(); ++iter)
+    {
+      const GroupInfo& group_info = iter->second;
+
+      xmlpp::Element* nodeGroup = nodeGroups->add_child("group");
+      nodeGroup->set_attribute("name", group_info.m_name);
+      set_node_attribute_value_as_bool(nodeGroup, "developer", group_info.m_developer);
+
+      //The privilieges for each table, for this group:
+      for(GroupInfo::type_map_table_privileges::const_iterator iter = group_info.m_map_privileges.begin(); iter != group_info.m_map_privileges.end(); ++iter)
+      {
+        xmlpp::Element* nodeTablePrivs = nodeGroups->add_child("table_privs");
+
+        set_node_attribute_value(nodeTablePrivs, "table_name", iter->first);
+
+        const Privileges& privs = iter->second;
+        set_node_attribute_value_as_bool(nodeTablePrivs, "priv_view", privs.m_view);
+        set_node_attribute_value_as_bool(nodeTablePrivs, "priv_edit", privs.m_edit);
+        set_node_attribute_value_as_bool(nodeTablePrivs, "priv_create", privs.m_create);
+        set_node_attribute_value_as_bool(nodeTablePrivs, "priv_delete", privs.m_delete);
+      }
+    }
+
   }
-  
+
   return Bakery::Document_XML::save_before();  
 }
 
@@ -1005,7 +1218,7 @@ void Document_Glom::set_database_title(const Glib::ustring& title)
     set_modified();
   }
 }
-  
+
 Glib::ustring Document_Glom::get_name() const
 {
   //Show the database title in the window title bar:
@@ -1014,3 +1227,45 @@ Glib::ustring Document_Glom::get_name() const
   else
     return m_database_title;
 }
+
+Document_Glom::type_list_groups Document_Glom::get_groups() const
+{
+  type_list_groups result;
+  for(type_map_groups::const_iterator iter = m_groups.begin(); iter != m_groups.end(); ++iter)
+  {
+    result.push_back(iter->second);
+  }
+
+  return result;
+}
+
+///This adds the group if necessary.
+void Document_Glom::set_group(GroupInfo& group)
+{
+  type_map_groups::iterator iter = m_groups.find(group.m_name);
+  if(iter == m_groups.end())
+  {
+    //Add it if necesary:
+    m_groups[group.m_name] = group;
+    set_modified();
+  }
+  else
+  {
+    const GroupInfo this_group = iter->second;
+    if(this_group != group)
+    {
+      iter->second = group;
+      set_modified();
+    }
+  }
+}
+
+void Document_Glom::remove_group(const Glib::ustring& group_name)
+{
+  type_map_groups::iterator iter = m_groups.find(group_name);
+  if(iter != m_groups.end())
+  {
+    m_groups.erase(iter);
+    set_modified();
+  }
+}
