@@ -21,6 +21,7 @@
 #include "frame_glom.h"
 #include "application.h"
 #include "appstate.h"
+#include "mode_design/users/dialog_groups_list.h"
 #include <libintl.h>
 
 Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
@@ -43,7 +44,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   refGlade->get_widget_derived("vbox_mode", m_pBox_Mode);
 
   //m_pLabel_Mode->set_text(gettext("No database selected.\n Use the Navigation menu, or open a previous Glom document."));
-  
+
   //Load the Glade file and instantiate its widgets to get the dialog stuff:
 
   try
@@ -57,7 +58,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
     std::cerr << ex.what() << std::endl;
   }
 
-  
+
   try
   {
     Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "box_navigation_tables");
@@ -105,8 +106,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   {
     std::cerr << ex.what() << std::endl;
   }
-  
-  
+
 
   m_Mode = MODE_None;
   m_Mode_Previous = MODE_None;
@@ -134,7 +134,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   add_view(m_pDialogConnection); //Also a composite view.
   add_view(&m_Notebook_Data); //Also a composite view.
   add_view(&m_Notebook_Find); //Also a composite view.
-  
+
   on_userlevel_changed(AppState::USERLEVEL_OPERATOR); //A default to show before a document is created or loaded.
   show_all();
 }
@@ -171,7 +171,6 @@ Frame_Glom::~Frame_Glom()
     m_pDialog_Relationships = 0;
   }
 
-  
 }
 
 void Frame_Glom::set_databases_selected(const Glib::ustring& strName)
@@ -298,7 +297,7 @@ void Frame_Glom::show_table(const Glib::ustring& strTableName)
   }
 
   show_table_title();
-  
+
   show_all();
 }
 
@@ -309,16 +308,25 @@ void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction
     Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
     if(document)
     {
-      //Avoid double signals:
-      //if(document->get_userlevel() != AppState::USERLEVEL_DEVELOPER)
-      bool test = document->set_userlevel(AppState::USERLEVEL_DEVELOPER);
-        
+
+      //Check whether the current user has developer privileges:
+      const ConnectionPool* connection_pool = ConnectionPool::get_instance();
+      bool test = get_user_is_in_group(connection_pool->get_user(), GLOM_STANDARD_GROUP_NAME_DEVELOPER);
+      if(test)
+      {
+        //Avoid double signals:
+        //if(document->get_userlevel() != AppState::USERLEVEL_DEVELOPER)
+        test = document->set_userlevel(AppState::USERLEVEL_DEVELOPER);
+      }
+
       //If this was not possible then revert the menu:
       if(!test)
       {
-        Gtk::MessageDialog dialog(gettext("Developer mode is not available. Check that you have sufficient database access rights and that the glom file is not read-only."), Gtk::MESSAGE_WARNING);
+        Gtk::MessageDialog dialog(gettext("<b>Developer mode not available.</b>"), true, Gtk::MESSAGE_WARNING);
+        dialog.set_secondary_text(gettext("Developer mode is not available. Check that you have sufficient database access rights and that the glom file is not read-only."));
+        dialog.set_transient_for(*get_app_window());
         dialog.run();
-        
+
         //This causes an endless loop, but it is not recursive so we can't block it.
         //TODO: Submit GTK+ bug.
         //action->set_active(false);
@@ -400,7 +408,7 @@ void Frame_Glom::do_menu_Navigate_Table(bool open_default)
     {
       default_table_name = get_document()->get_default_table();
     }
-    
+
     if(!default_table_name.empty())
     {
       //Show the default table, and let the user navigate to another table manually if he wants:
@@ -479,7 +487,6 @@ void Frame_Glom::on_userlevel_changed(AppState::userlevels userlevel)
   if(userlevel == AppState::USERLEVEL_DEVELOPER)
     user_level_name = gettext("Developer");
 
-    
   if(m_pLabel_userlevel)
     m_pLabel_userlevel->set_text(user_level_name);
 
@@ -514,22 +521,22 @@ void Frame_Glom::update_table_in_document_from_database()
   //The database should never change without the knowledge of the document anyway, so this should be unnecessary.
 
   //TODO_performance: There are a lot of temporary Field and FieldAttributes instances here, with a lot of string copying.
-  
+
   //For instance, changed field details, or new fields, or removed fields.
   typedef Box_DB_Table::type_vecFields type_vecFields;
 
   //Get the fields information from the database:
   Base_DB::type_vecFields fieldsDatabase = Base_DB::get_fields_for_table_from_database(m_strTableName);
-  
+
   Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
   if(pDoc)
   {
     bool document_must_to_be_updated = false;
-    
+
     //Get the fields information from the document.
     //and add to, or update Document's list of fields:
     type_vecFields fieldsDocument = pDoc->get_table_fields(m_strTableName);
-    
+
     for(Base_DB::type_vecFields::const_iterator iter = fieldsDatabase.begin(); iter != fieldsDatabase.end(); ++iter)
     {
       const Field& field_database = *iter;
@@ -578,7 +585,7 @@ void Frame_Glom::update_table_in_document_from_database()
         document_must_to_be_updated = true; //Something changed.
       }
     }
-          
+
     if(document_must_to_be_updated)
       pDoc->set_table_fields(m_strTableName, fieldsActual);
   }
@@ -640,7 +647,30 @@ void Frame_Glom::on_menu_developer_relationships()
 
 void Frame_Glom::on_menu_developer_users()
 {
+  Dialog_GroupsList* dialog = 0;
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_groups");
 
+    refXml->get_widget_derived("window_groups", dialog);
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  dialog->set_transient_for(*get_app_window());
+
+  add_view(dialog); //Give it access to the document.
+  dialog->load_from_document(); //Update the UI now that it has the document.
+
+  dialog->run();
+  delete dialog;
+
+  //Update the Details and List layouts, in case the permissions have changed:
+  //TODO: Also update them somehow if another user has changed them,
+  //or respond to the failed SQL nicely.
+  show_table(m_strTableName);
 }
 
 void Frame_Glom::on_menu_developer_layout()
@@ -674,6 +704,7 @@ bool Frame_Glom::connection_request_password_and_attempt()
   {
     //Ask for connection details:
     m_pDialogConnection->load_from_document(); //Get good defaults.
+    m_pDialogConnection->set_transient_for(*get_app_window());
     int response = m_pDialogConnection->run();
     m_pDialogConnection->hide();
 
@@ -688,10 +719,11 @@ bool Frame_Glom::connection_request_password_and_attempt()
       catch(const ExceptionConnection& ex)
       {
         g_warning("Frame_Glom::connection_request_password_and_attempt(): caught exception.");
-         
+
         if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER)
         {
           //Warn the user, and let him try again:
+          m_pDialogConnectionFailed->set_transient_for(*get_app_window());
           int response = m_pDialogConnectionFailed->run();
           m_pDialogConnectionFailed->hide();
 
@@ -707,7 +739,7 @@ bool Frame_Glom::connection_request_password_and_attempt()
           return false;
         }
       }
-      
+
       //Try again.
     }
     else
@@ -751,10 +783,10 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, bool reques
     catch(const ExceptionConnection& ex)
     {
       g_warning("debug Frame_Glom::create_database() Connection failed.");
- 
+
       return false;
     }
-      
+
     if(sharedconnection)
     {
       Bakery::BusyCursor(*get_app_window());
@@ -783,6 +815,7 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, bool reques
             std::cerr << ex.what() << std::endl;
           }
 
+          //TODO: dialog->set_transient_for(*get_app_window());
           dialog->run();
           delete dialog;
           return false; //Failed. //TODO: Allow the user to try with a different user name?
@@ -790,13 +823,12 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, bool reques
       }
     }
   }
- 
+
   return false;
 }
 
 
 
- 
 
 
 
